%pop_BootstrapMSNumber() Reorder microstate maps based on a mean template
%
% Usage: >> com = pop_BootstrapMSNumber(AllEEG,SetToSort, IgnorePolarity, LearningsetSize)
%
% EEG lab specific:
%
%   "AllEEG" 
%   -> AllEEG structure with all the EEGs that may be analysed
%   
%   "SetToSort" 
%   -> Index of selected EEGs. If more than one EEG is selected, the analysis
%      will be limited to those, if not, the user is asked.
%
%   "IgnorePolarity"
%   -> Ignore the polarity of the maps to be sorted   
%
%   "LearningsetSize"
%   -> Percent samples used for the learning set
% Output:
%
%   "com"
%   -> Command necessary to replicate the computation
%
% Author: Thomas Koenig, University of Bern, Switzerland, 2016
%
% Copyright (C) 2016 Thomas Koenig, University of Bern, Switzerland, 2016
% thomas.koenig@puk.unibe.ch
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%
function com = pop_MS_Silhouette(AllEEG,SetToSort, IgnorePolarity)

    com = '';
    
%    if numel(EEG) > 1
%        errordlg2('pop_findMSTemplates() currently supports only a single EEG as input');
%        return;
%    end
    if nargin < 2 
        SetToSort = [];
    end

    if nargin < 3  IgnorePolarity = true;            end %#ok<*SEPEX>
    
    nonempty = find(cellfun(@(x) isfield(x,'msinfo'), num2cell(AllEEG)));
    HasChildren = cellfun(@(x) isfield(x,'children'), {AllEEG.msinfo});
    nonemptyInd  = nonempty(~HasChildren);
    
    if numel(SetToSort) < 3 
        AvailableSets = {AllEEG(nonemptyInd).setname};
  
        res = inputgui( 'geometry', {1 1 1 1}, 'geomvert', [1 1 4 1], 'uilist', { ...
            { 'Style', 'text', 'string', 'Choose sets for sorting'} ...
            { 'Style', 'text', 'string', 'Use ctrlshift for multiple selection'} ...
            { 'Style', 'listbox', 'string', AvailableSets, 'tag','SelectSets' ,'Min', 0, 'Max',2} ...
            { 'Style', 'checkbox', 'string', 'No polarity','tag','Ignore_Polarity' ,'Value', IgnorePolarity }  ...
            });
     
        if isempty(res)
            return; 
        end
        IgnorePolarity = res{4};
        SelectedSet = nonemptyInd(res{1});

    else
        if nargin < 3
            res = inputgui( 'geometry', {1}, 'geomvert', 1, 'uilist', { ...
                { 'Style', 'checkbox', 'string', 'No polarity','tag','Ignore_Polarity' ,'Value', IgnorePolarity }  ...
                });
        
            if isempty(res); return; end
            IgnorePolarity = res{3};
        end    
        SelectedSet = SetToSort;
    end

    if numel(SelectedSet) < 1
        errordlg2('You must select at least one set of microstate maps','Sort microstate classes');
        return
    end
    
   
    eegout = pop_CombMSTemplates(AllEEG, SelectedSet, 0, 0, [], IgnorePolarity);
    eegout.msinfo = rmfield(eegout.msinfo,'children');
        
    [~,SortedSet] = pop_SortMSTemplates(AllEEG,SelectedSet, false,eegout, IgnorePolarity);
    pidx = 1;
    nSubjects = numel(SelectedSet);
    for nc = eegout.msinfo.ClustPar.MinClasses:eegout.msinfo.ClustPar.MaxClasses
        SortedMaps = cellfun(@(x) SortedSet(x).msinfo.MSMaps(nc).Maps,num2cell(1:nSubjects),'UniformOutput',false);
        [nmaps,nchannels] = size(SortedMaps{1,1});
        SortedMaps = NormDimL2(reshape(cell2mat(SortedMaps),[nmaps,nchannels,nSubjects]),2);
        Clusters = repmat((1:nc),nSubjects,1);
        Clusters = reshape(Clusters,1,nmaps * nSubjects);
        SortedMapsShifted = shiftdim(SortedMaps,1);

        SortedMapsShifted = reshape(SortedMapsShifted,nchannels,nmaps * nSubjects);

        distfun = @(XI,XJ)(1-(corr(XI',XJ').^2 ));
        
        subplot(1,(eegout.msinfo.ClustPar.MaxClasses - eegout.msinfo.ClustPar.MinClasses+1),pidx);
        figure
        [s,h] = silhouette(SortedMapsShifted',Clusters,distfun);
        axis([-1 1 1 nSubjects * eegout.msinfo.ClustPar.MaxClasses]);
        
        title(sprintf('Mean s: %f',mean(s)));
        pidx = pidx + 1;
        
    end
    txt = sprintf('%i ',SelectedSet);
    txt(end) = [];
    
        
    com = sprintf('com = pop_MS_Silhouette(%s, [%s], %i);', inputname(1),txt,IgnorePolarity); 
end
